#!/usr/bin/python
import math
import pdb
from OpenGL.GL import *
from OpenGL.GLU import *
from PyQt4 import QtGui
from PyQt4.QtOpenGL import *
from PyQt4.Qt import Qt
from PyQt4 import QtCore
from PyNGL import *
import sys
import random

class GLWindow(QGLWidget):

	def __init__(self, parent):
		QGLWidget.__init__(self, parent)
		self.setMinimumSize(1024, 720)
		self.m_spinYFace = 0
		self.m_spinXFace = 0
		self.m_origX = 0
		self.m_origY = 0
		self.m_transformStack=TransformStack()


	def mousePressEvent (self,  _event) :

		# this method is called when the mouse button is pressed in this case we
		# store the value where the maouse was clicked (x,y) and set the Rotate flag to true
		if(_event.button() == Qt.LeftButton) :
			self.m_origX = _event.x();
			self.m_origY = _event.y();
			self.m_rotate =True;



	def mouseMoveEvent ( self,_event ) :
		# note the method buttons() is the button state when event was called
		# this is different from button() which is used to check which button was
		# pressed when the mousePress/Release event is generated
		if(self.m_rotate and _event.buttons() == Qt.LeftButton) :
			self.m_spinYFace = ( self.m_spinYFace + (_event.x() - self.m_origX) ) % 360
			self.m_spinXFace = ( self.m_spinXFace + (_event.y() - self.m_origY) ) % 360
			self.m_origX = _event.x();
			self.m_origY = _event.y();
			# re-draw GL
			self.updateGL();

	def mouseReleaseEvent (self,  _event) :

		# this event is called when the mouse button is released
		# we then set Rotate to false
		if (_event.button() == Qt.LeftButton) :
			self.m_rotate=False


	def paintGL(self):
			'''
			Drawing routine
			'''
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)


			trans=Transformation()
			# set the mouse rotation
			trans.setRotation(self.m_spinXFace,self.m_spinYFace,0)
			# set this in the TX stack
			self.m_transformStack.setGlobal(trans);

			shader=ShaderManager.instance();
			shader.setShaderParamFromMatrix("gl3xTest","ModelMatrix",self.m_transformStack.getCurrAndGlobal().getTransposeMatrix());

			vbo=VBOPrimitives.instance()
			vbo.draw("teapot")


			glFlush()



	def resizeGL(self, w, h):
			'''
			Resize the GL window
			'''
			glViewport(0, 0, w, h)
			self.m_cam.setShape(45,float(w/h),0.5,10,CAMERAPROJECTION.PERSPECTIVE)





	def initializeGL(self):
			'''
			Initialize GL
			'''
			# set viewing projection
			ngl=NGLInit.instance()
			ngl.initGlew()
			glClearColor(0.4, 0.4, 0.4, 1.0)
			glClearDepth(1.0)
			From=Vector(0,1,1)
			To=Vector(0,0,0)
			Up=Vector(0,1,0)
			self.m_cam=Camera(From,To,Up,CAMERAPROJECTION.PERSPECTIVE)
			self.m_cam.setShape(45,1024.0/720.0,0.5,10,CAMERAPROJECTION.PERSPECTIVE);
			glEnable(GL_LIGHTING)
			glShadeModel(GL_SMOOTH)
			glEnable(GL_DEPTH_TEST)
			glEnable(GL_NORMALIZE)
			glClearColor(0.5,0.5,0.5,1.0)

			self.shader=ShaderManager.instance()
			# load a frag and vert shaders
			self.shader.loadShader("gl3xTest","Vertex.vs","Fragment.fs","",True)
			# set this as the active shader
			self.shader.useShader("gl3xTest");
			# now pass the modelView and projection values to the shader
			self.shader.setShaderParamFromMatrix("gl3xTest","ViewMatrix",self.m_cam.getModelView())
			self.shader.setShaderParamFromMatrix("gl3xTest","projectionMatrix",self.m_cam.getProjection())

			m=Material(STDMAT.GOLD)
			m.use()
			self.Light0 = Light(Vector(5,12,0,1),Colour(1,1,1),Colour(1,1,1),LIGHTMODES.LIGHTLOCAL)
			self.Light0.enable()



#
# You don't need anything below this
class NGLOpenGLDemo(QtGui.QMainWindow):

	def __init__(self):
		QtGui.QMainWindow.__init__(self)
		self.widget = GLWindow(self)
		self.setCentralWidget(self.widget)


	def keyPressEvent(self ,_event) :
		INC=0.1

		if _event.key() == Qt.Key_Q	:
			sys.exit()
		elif _event.key() == Qt.Key_W :
			glPolygonMode(GL_FRONT_AND_BACK,GL_LINE)
		elif _event.key() == Qt.Key_S :
			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
		elif _event.key() == Qt.Key_Up :
			self.widget.m_cam.Move(0.0,-INC,0.0)
		elif _event.key() == Qt.Key_Down :
			self.widget.m_cam.Move(0.0,INC,0.0)
		elif _event.key() == Qt.Key_Left :
			self.widget.m_cam.Move(INC,0.0,0.0)
		elif _event.key() == Qt.Key_Right :
			self.widget.m_cam.Move(-INC,0.0,0.0)

		elif _event.key() == Qt.Key_I :
			self.widget.m_cam.Move(0.0,0.0,INC)
		elif _event.key() == Qt.Key_O :
			self.widget.m_cam.Move(0.0,0.0,-INC)

		self.widget.updateGL()


if __name__ == '__main__':
	app = QtGui.QApplication(['Simple NGL Python Demo'])
	window = NGLOpenGLDemo()
	window.setWindowTitle("PyNGL Demo")
	window.show()
	app.exec_()